/*
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ----------------------sce remarks -------------------------------------------
 * as in the Errata the SSI1 only works at Legacy Mode (single SPI)
 * Also in quadSPI the Dat0 - Dat1 lines are exchanged
 * SSI0 and SSI2 lines are used otherwise on the EK-TM4C1294 board
 * so only SSI3 remains.
 * with UDMA
 */

// XDCtools Header files
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Types.h>
#include <xdc/runtime/Error.h>
// BIOS Header files
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/hal/Hwi.h>
// TI-RTOS Header files
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>
// TI-TIVAWARE Driverlib
#include <driverlib/ssi.h>
#include <driverlib/udma.h>
#include "inc/hw_memmap.h"
#include "inc/hw_ssi.h"
#include "inc/hw_types.h"
#include "inc/hw_ints.h"
// Example/Board Header files
#include "Board.h"

#define SSI_BUFFER_SIZE   270
#define TASKSTACKSIZE     768
#define MSG_LENGTH        28

Task_Struct task0Struct;
Char task0Stack[TASKSTACKSIZE];
//Semaphore_Handle TxFifoNotFull;
//Semaphore_Struct sem0Struct;
Hwi_Handle SSI3IH;

UInt8  ui8SSITxBuf[SSI_BUFFER_SIZE];  // Tx Buffer
//UInt8  ui8SSIRxBuf[SSI_BUFFER_SIZE];  // Rx Buffer
UInt32 ui32uDMAErrCount = 0;


//*****************************************************************************
// SSI3 Interrupt Handler
//************uDMAErrorHandler*****************************************************************

void SSI3IntHandler(void)
{
    HWREG(SSI3_BASE + SSI_O_ICR) = SSI_TXEOT;  // clear irq flag
}

void masterTaskFxn (UArg arg0, UArg arg1) {

    Types_FreqHz freq;
    UInt16 i;

    for(i=0; i<SSI_BUFFER_SIZE; i++) {
        ui8SSITxBuf[i] = (UInt8)i;  // fill txBuff with pattern
    }

    // configure SSI3
    BIOS_getCpuFreq(&freq);     // spi initialise 1MBaud, 8bit, master
    SSIConfigSetExpClk(SSI3_BASE, freq.lo, SSI_FRF_MOTO_MODE_0, SSI_MODE_MASTER, 25000000, 8);
    SSIAdvModeSet(SSI3_BASE, SSI_ADV_MODE_QUAD_WRITE);  // set to quad write mode
    SSIAdvFrameHoldEnable(SSI3_BASE);   // hold FSS

    // configure UDMA
    SSIDMAEnable(SSI3_BASE, SSI_DMA_TX);    // Enable the uDMA interface for TX channel.
    //SSIDMAEnable(SSI3_BASE, SSI_DMA_RX | SSI_DMA_TX);    // Enable the uDMA interface for both TX and RX channels.

    // uDMA SSI3 RX// signal end of transfer
    // Clear attributes for uDMA SSI0RX channel, should already be done by default.
    //uDMAChannelAttributeDisable(UDMA_CHANNEL_SSI3RX, UDMA_ATTR_USEBURST | UDMA_ATTR_ALTSELECT |
    //                  (UDMA_ATTR_HIGH_PRIORITY | UDMA_ATTR_REQMASK));
    // Configure the control parameters for the primary control structure for the SSI3RX channel.
    //uDMAChannelControlSet(UDMA_CHANNEL_SSI3RX | UDMA_PRI_SELECT,
    //                      UDMA_SIZE_8 | UDMA_SRC_INC_NONE | UDMA_DST_INC_8 | UDMA_ARB_4);
    // Set up the transfer parameters for the SSI3RX Channel
    //uDMAChannelTransferSet(UDMA_CHANNEL_SSI3RX | UDMA_PRI_SELECT, UDMA_MODE_BASIC,
    //            (void *)(SSI3_BASE + SSI_O_DR), ui8SSIRxBuf, sizeof(ui8SSIRxBuf));

    //uDMA SSI3 TX
    // Clear attributes for uDMA SSI3TX channel, should already be done by default.
    uDMAChannelAttributeDisable(15, UDMA_ATTR_ALTSELECT | UDMA_ATTR_HIGH_PRIORITY | UDMA_ATTR_REQMASK);
    // Set the USEBURST attribute for the uDMA SSI3TX channel.  This will
    // force the controller to always use a burst when transferring data from the TX buffer
    uDMAChannelAttributeEnable(15, UDMA_ATTR_USEBURST);
    // Configure the control parameters for the SSI3 TX.
    uDMAChannelControlSet(15, UDMA_SIZE_8 | UDMA_SRC_INC_8 | UDMA_DST_INC_NONE | UDMA_ARB_4);
    // Set up the transfer parameters for the uDMA SSI3 TX channel.
    uDMAChannelTransferSet(15, UDMA_MODE_BASIC, ui8SSITxBuf, (void *)(SSI3_BASE + SSI_O_DR), MSG_LENGTH-1 );
    // Now both the uDMA SSI0 TX and RX channels are primed to start a
    // transfer.  As soon as the channels are enabled, the peripheral will
    // issue a trDMAErrorHandleransfer request and the data transfers will begin.
    //uDMAChannelEnable(UDMA_CHANNEL_SSI3RX);
    uDMAChannelAssign(UDMA_CH15_SSI3TX);    // assign channel mapping
    uDMAChannelEnable(15);              // fill SSI3 Tx Fifo
//    SSIIntEnable(SSI3_BASE, SSI_DMATX | SSI_DMARX); // Enable the SSI3 DMA TX/RX interrupts.
    SSIEnable(SSI3_BASE);               // enable the SSI3, now all bytes in txfifo will be sent
    SSIIntEnable(SSI3_BASE, SSI_TXEOT); // Enable the SSI3 DMA TX complete interrupt.

//    while(!HWREG(SSI3_BASE + SSI_O_RIS) & SSI_RIS_DMATXRIS);
    while(uDMAChannelSizeGet(15) > 8);  // no other way found to detect end of DMA!!
    SSIAdvDataPutFrameEnd(SSI3_BASE, ui8SSITxBuf[MSG_LENGTH - 1]);  // this finally stops the
                                    // transfer and causes the irq to occur

    System_printf("Done\n");
    System_flush();
    Hwi_delete(&SSI3IH);
    uDMAChannelDisable(15);
    SSIDisable(SSI3_BASE);               // switch off SSI3

}

int main(void)
{
    Task_Params taskParams;         // Construct BIOS objects
    Hwi_Params hwiParams;
    Error_Block eb;

    Board_initGeneral();        // Call board init functions.
    Board_initGPIO();
    Board_initSPI();

    Hwi_Params_init(&hwiParams);
    hwiParams.priority = -1;
    SSI3IH = Hwi_create(INT_SSI3,    (Hwi_FuncPtr)(SSI3IntHandler),   &hwiParams, &eb);

    Task_Params_init(&taskParams);    // Construct Task thread
    taskParams.priority = 1;
    taskParams.stackSize = TASKSTACKSIZE;
    taskParams.stack = &task0Stack;
    Task_construct(&task0Struct, (Task_FuncPtr)masterTaskFxn, &taskParams, NULL);

    GPIO_write(Board_LED0, Board_LED_ON);   // Turn on user LED
    BIOS_start();    // Start BIOS
    return (0);
}

