/ * ----------------------sce remarks -------------------------------------------
 * as in the Errata the SSI1 only works at Legacy Mode (single SPI)
 * Also in quadSPI the Dat0 - Dat1 lines are exchanged
 * SSI0 and SSI2 lines are used otherwise on the EK-TM4C1294 board
 * so only SSI3 remains.
 * with UDMA
 */

// XDCtools Header files
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Types.h>
#include <xdc/runtime/Error.h>
// BIOS Header files
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/hal/Hwi.h>
// TI-RTOS Header files
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>
// TI-TIVAWARE Driverlib
#include <driverlib/ssi.h>
#include <driverlib/udma.h>
#include "inc/hw_memmap.h"
#include "inc/hw_ssi.h"
#include "inc/hw_types.h"
#include "inc/hw_ints.h"
// Example/Board Header files
#include "Board.h"

#define SSI_BUFFER_SIZE   270
#define TASKSTACKSIZE     768
#define MSG_LENGTH        28

Task_Struct task0Struct;
Char task0Stack[TASKSTACKSIZE];
Hwi_Handle SSI3IH;

UInt8  ui8SSITxBuf[SSI_BUFFER_SIZE];  // Tx Buffer
UInt32 ui32uDMAErrCount = 0;


//*****************************************************************************
// SSI3 Interrupt Handler
//************uDMAErrorHandler*****************************************************************
void SSI3IntHandler(void)
{
    UInt32 ui32Status;

    ui32Status = SSIIntStatus(SSI3_BASE, 1);
    SSIIntClear(SSI3_BASE, ui32Status);
    // signal end of transfer
    HWREG(SSI3_BASE + SSI_O_CR1) |= SSI_CR1_EOM;
    HWREG(SSI3_BASE + SSI_O_DR) = ui8SSITxBuf[MSG_LENGTH - 1];
}


void masterTaskFxn (UArg arg0, UArg arg1) {

    Types_FreqHz freq;
    UInt16 i;

    for(i=0; i<SSI_BUFFER_SIZE; i++) {
        ui8SSITxBuf[i] = (UInt8)i;  // fill txBuff with pattern
    }

    // configure SSI3
    BIOS_getCpuFreq(&freq);     // spi initialise 1MBaud, 8bit, master
    SSIConfigSetExpClk(SSI3_BASE, freq.lo, SSI_FRF_MOTO_MODE_0, SSI_MODE_MASTER, 10000000, 8);
    SSIAdvModeSet(SSI3_BASE, SSI_ADV_MODE_QUAD_WRITE);  // set to quad write mode
    SSIAdvFrameHoldEnable(SSI3_BASE);   // hold FSS

    // configure UDMA
    SSIDMAEnable(SSI3_BASE, SSI_DMA_TX);    // Enable the uDMA interface for TX channel.
    //uDMA SSI3 TX
    // Clear attributes for uDMA SSI3TX channel, should already be done by default.
    uDMAChannelAttributeDisable(15, UDMA_ATTR_ALTSELECT | UDMA_ATTR_HIGH_PRIORITY | UDMA_ATTR_REQMASK);
    // Set the USEBURST attribute for the uDMA SSI3TX channel.  This will
    // force the controller to always use a burst when transferring data from the TX buffer
    uDMAChannelAttributeEnable(15, UDMA_ATTR_USEBURST);
    // Configure the control parameters for the SSI3 TX.
    uDMAChannelControlSet(15, UDMA_SIZE_8 | UDMA_SRC_INC_8 | UDMA_DST_INC_NONE | UDMA_ARB_4);
    // Set up the transfer parameters for the uDMA SSI3 TX channel.
    uDMAChannelTransferSet(15, UDMA_MODE_BASIC, ui8SSITxBuf, (void *)(SSI3_BASE + SSI_O_DR), MSG_LENGTH-1 );
    uDMAChannelAssign(UDMA_CH15_SSI3TX);    // assign channel mapping
    uDMAChannelEnable(15);              // fill SSI3 Tx Fifo
    SSIIntEnable(SSI3_BASE, SSI_DMATX | SSI_DMARX); // Enable the SSI3 DMA TX/RX interrupts.
    SSIIntEnable(SSI3_BASE, SSI_DMATX); // Enable the SSI3 DMA TX interrupt.
    SSIEnable(SSI3_BASE);               // enable the SSI3, now all bytes in txfifo will be sent

    System_printf("Done\n");
    Hwi_delete(&SSI3IH);
    uDMAChannelDisable(15);
    SSIDisable(SSI3_BASE);               // switch off SSI3
    System_flush();
}

int main(void)
{
    Task_Params taskParams;         // Construct BIOS objects
    Hwi_Params hwiParams;
    Error_Block eb;

    Board_initGeneral();        // Call board init functions.
    Board_initGPIO();
    Board_initSPI();

    Hwi_Params_init(&hwiParams);
    hwiParams.priority = -1;
    SSI3IH = Hwi_create(INT_SSI3,    (Hwi_FuncPtr)(SSI3IntHandler),   &hwiParams, &eb);

    Task_Params_init(&taskParams);    // Construct Task thread
    taskParams.priority = 1;
    taskParams.stackSize = TASKSTACKSIZE;
    taskParams.stack = &task0Stack;
    Task_construct(&task0Struct, (Task_FuncPtr)masterTaskFxn, &taskParams, NULL);

    GPIO_write(Board_LED0, Board_LED_ON);   // Turn on user LED
    BIOS_start();    // Start BIOS
    return (0);
}

