/*
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ----------------------sce remarks -------------------------------------------
 * as in the Errata the SSI1 only works at Legacy Mode (single SPI)
 * Also in quadSPI the Dat0 - Dat1 lines are exchanged
 * SSI0 and SSI2 lines are used otherwise on the EK-TM4C1294 board
 * so only SSI3 remains.
 */

// XDCtools Header files
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Types.h>
#include <xdc/runtime/Error.h>
// BIOS Header files
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/hal/Hwi.h>
// TI-RTOS Header files
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>
// TI-TIVAWARE Driverlib
#include <driverlib/ssi.h>
#include "inc/hw_memmap.h"
#include "inc/hw_ssi.h"
#include "inc/hw_types.h"
#include "inc/hw_ints.h"
// Example/Board Header files
#include "Board.h"

#define SPI_MSG_LENGTH    28
#define SPI_MSG_LENGTH_1  27

#define TASKSTACKSIZE     768

Task_Struct task0Struct;
Char task0Stack[TASKSTACKSIZE];
UInt8 masterTxBuffer[SPI_MSG_LENGTH];
Semaphore_Handle TxFifoNotFull;
Semaphore_Struct sem0Struct;
Hwi_Handle TxFifoHwi;



void TxFifoNotFullCallBack() {
    SSIIntClear     (SSI3_BASE, SSI_TXFF);   // clear irq flag
    SSIIntDisable   (SSI3_BASE, SSI_TXFF);   // disable TXFifo half-full or less interrupt

    Semaphore_post(TxFifoNotFull);      // semaphore
}


void masterTaskFxn (UArg arg0, UArg arg1) {

//    SPI_Handle masterSpi;
//    SPI_Transaction masterTransaction;
//    bool transferOK;
//    union {
//        Semaphore_Params       semParams;
//        Hwi_Params             hwiParams;
//    } paramsUnion;

    Types_FreqHz freq;
    UInt8 i;

    /*
    masterSpi = SPI_open(Board_SPI0, &mSpiParams);      // Initialise SPI, DMA, HWI
    if (masterSpi == NULL) System_abort("Error initializing SPI\n");
    else System_printf("SPI master initialized\n");

    masterTransaction.count = SPI_MSG_LENGTH;    // Initialise master SPI transaction structure
    masterTransaction.txBuf = (Ptr)masterTxBuffer;
    masterTransaction.rxBuf = NULL;

    transferOK = SPI_transfer(masterSpi, &masterTransaction);   // Initiate SPI transfer
    if(!transferOK) System_printf("Unsuccessful master SPI tx");
    SPI_close(masterSpi);    // Deinitialize SPI
    */

    for(i=0; i<SPI_MSG_LENGTH; i++) {
        masterTxBuffer[i] = i;
    }
    BIOS_getCpuFreq(&freq);     // spi initialise 1MBaud, 8bit, master
    SSIConfigSetExpClk(SSI3_BASE, freq.lo, SSI_FRF_MOTO_MODE_0, SSI_MODE_MASTER, 20000000, 8);
    SSIAdvModeSet(SSI3_BASE, SSI_ADV_MODE_QUAD_WRITE);  // set to quad write mode

    SSIAdvFrameHoldEnable(SSI3_BASE);   // hold FSS
    SSIEnable(SSI3_BASE);               // enable the SSI3, now all bytes in txfifo will be sent
    for(i=0; i<SPI_MSG_LENGTH_1; i++) {
        HWREG(SSI3_BASE + SSI_O_DR) = masterTxBuffer[i];    // this is faster
        /*
        if(!SSIDataPutNonBlocking(SSI3_BASE, masterTxBuffer[i])) {  // put data in TxFifo
            // TxFifo is full, unable to store more data....
            SSIIntEnable (SSI3_BASE, SSI_TXFF);                 // enable TXFifo half-full or less interrupt
            Semaphore_pend(TxFifoNotFull, BIOS_WAIT_FOREVER);   // block task until TxFifo is half empty
            SSIDataPut(SSI3_BASE, masterTxBuffer[i]);           // then resend data
        }
        */
    }
    HWREG(SSI3_BASE + SSI_O_CR1) |= SSI_CR1_EOM;
    HWREG(SSI3_BASE + SSI_O_DR) = masterTxBuffer[SPI_MSG_LENGTH - 1];

//    SSIAdvDataPutFrameEnd(SSI3_BASE, masterTxBuffer[SPI_MSG_LENGTH - 1]); // signals end of transfer
    System_printf("Done\n");
    Hwi_delete(&TxFifoHwi);
    SSIDisable(SSI3_BASE);               // switch off SSI3
    System_flush();
}

int main(void)
{
    Task_Params taskParams;         // Construct BIOS objects
    Semaphore_Params semParams;
    Hwi_Params hwiParams;
    Error_Block eb;

    Board_initGeneral();        // Call board init functions.
    Board_initGPIO();
    Board_initSPI();

    // Construct a Semaphore object to be used as a resource lock, inital count 0
    Semaphore_Params_init(&semParams);
    Semaphore_construct(&sem0Struct, 0, &semParams);
    TxFifoNotFull = Semaphore_handle(&sem0Struct); // Obtain instance handle

    Hwi_Params_init(&hwiParams);
    hwiParams.priority = -1;
    TxFifoHwi = Hwi_create(INT_SSI3, (Hwi_FuncPtr)(TxFifoNotFullCallBack), &hwiParams, &eb);

    Task_Params_init(&taskParams);    // Construct Task thread
    taskParams.priority = 1;
    taskParams.stackSize = TASKSTACKSIZE;
    taskParams.stack = &task0Stack;
    Task_construct(&task0Struct, (Task_FuncPtr)masterTaskFxn, &taskParams, NULL);

    GPIO_write(Board_LED0, Board_LED_ON);   // Turn on user LED
    BIOS_start();    // Start BIOS
    return (0);
}

